import React from 'react';
import { Animated, Dimensions, View } from 'react-native';

const window = Dimensions.get('window');

const ListLayout = <ItemType extends { key: string }>(props: {
    headerHeight: number;
    tabBarHeight: number;
    isFocused: boolean;
    onMomentumScrollBegin: any;
    onMomentumScrollEnd: any;
    onScrollEndDrag: any;
    scrollY: any;
    listArrRef: any;
    tabRoute: { key: string; title: string };
    list: Animated.WithAnimatedObject<ItemType[]>;
    renderCard: ({ item, index, loaded }: { item: ItemType; index: number; loaded: boolean }) => JSX.Element;
    ListHeaderComponent?: JSX.Element;
}) => {
    const {
        headerHeight,
        tabBarHeight,
        tabRoute,
        listArrRef,
        isFocused,
        onMomentumScrollBegin,
        onMomentumScrollEnd,
        onScrollEndDrag,
        scrollY,
        list,
        renderCard,
        ListHeaderComponent,
    } = props;

    return (
        <View style={{ flex: 1 }}>
            <Animated.FlatList
                ListHeaderComponent={ListHeaderComponent}
                ref={(ref) => {
                    let foundIndex = listArrRef.current.findIndex((e: any) => e.key === tabRoute.key);

                    if (foundIndex === -1) {
                        listArrRef.current.push({
                            key: tabRoute.key,
                            value: ref,
                        });
                    } else {
                        listArrRef.current[foundIndex] = {
                            key: tabRoute.key,
                            value: ref,
                        };
                    }
                }}
                data={list}
                renderItem={({ item, index }) => {
                    return renderCard({ item, index, loaded: false });
                }}
                keyExtractor={(item, index) => index.toString()}
                contentContainerStyle={{
                    paddingTop: headerHeight,
                    minHeight: window.height + headerHeight - tabBarHeight,
                }}
                scrollEventThrottle={16}
                onScroll={isFocused ? Animated.event([{ nativeEvent: { contentOffset: { y: scrollY } } }], { useNativeDriver: true }) : undefined}
                onMomentumScrollBegin={onMomentumScrollBegin}
                onMomentumScrollEnd={onMomentumScrollEnd}
                onScrollEndDrag={onScrollEndDrag}
                bounces={false}
            />
        </View>
    );
};

export default ListLayout;
